==> models_weebi-1.0.22.tar.gz: lib/src/extensions/tickets_purchasing_power.dart <==
// Project imports:

import 'package:models_weebi/src/common/ticket_type.dart';
import 'package:models_weebi/weebi_models.dart';

extension TicketsHerderPurchasingPowerEndDate on Iterable<TicketWeebi> {
  num herderTicketTypeEndDate(TicketType ticketType, String contactIdString,
      {DateTime? end}) {
    final filterByRange = (end != null)
        ? where((t) => t.status)
            .where((t) => '${t.contactId}' == contactIdString)
            .where((t) => t.date.isBefore(end) || t.date.isAtSameMomentAs(end))
        : where((t) => t.status)
            .where((t) => '${t.contactId}' == contactIdString);

    print('in filter by range');
    switch (ticketType) {
      case TicketType.sell:
        return filterByRange
            .where((t) => t.ticketType == TicketType.sell)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.sellDeferred:
        return filterByRange
            .where((t) => t.ticketType == TicketType.sellDeferred)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.sellCovered:
        return filterByRange
            .where((t) => t.ticketType == TicketType.sellCovered)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.spend:
        return filterByRange
            .where((t) => t.ticketType == TicketType.spend)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.spendDeferred:
        return filterByRange
            .where((t) => t.ticketType == TicketType.spendDeferred)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.spendCovered:
        return filterByRange
            .where((t) => t.ticketType == TicketType.spendCovered)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.wage:
        return filterByRange
            .where((t) => t.ticketType == TicketType.wage)
            .fold(0, (prev, element) => prev + element.total);
      case TicketType.unknown:
        print(
            'rhaaaa unknow ticket type in herderTicketTypeRange, not again !');
        return 0;
      default:
        return 0;
    }
  }

  num herderTicketTypeRangeDate(TicketType ticketType, String contactIdString,
      DateTime start, DateTime end) {
    final filterByRange = where((t) => t.status)
        .where((t) => '${t.contactId}' == contactIdString)
        .where((t) => t.date.isAfter(start) || t.date.isAtSameMomentAs(start))
        .where((t) => t.date.isBefore(end) || t.date.isAtSameMomentAs(end));

    switch (ticketType) {
      case TicketType.sell:
        return filterByRange
            .where((t) => t.ticketType == TicketType.sell)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.sellDeferred:
        return filterByRange
            .where((t) => t.ticketType == TicketType.sellDeferred)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.sellCovered:
        return filterByRange
            .where((t) => t.ticketType == TicketType.sellCovered)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.spend:
        return filterByRange
            .where((t) => t.ticketType == TicketType.spend)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.spendDeferred:
        return filterByRange
            .where((t) => t.ticketType == TicketType.spendDeferred)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.spendCovered:
        return filterByRange
            .where((t) => t.ticketType == TicketType.spendCovered)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.wage:
        return filterByRange
            .where((t) => t.ticketType == TicketType.wage)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.unknown:
        print(
            'rhaaaa unknow ticket type in herderTicketTypeRange, not again !');

        return 0;
      default:
        return 0;
    }
  }

  Map<TicketType, num> herderAllTicketTypeEndDate(String contactIdString,
      {DateTime? end}) {
    final filterByRange = (end != null)
        ? where((t) => t.status)
            .where((t) => '${t.contactId}' == contactIdString)
            .where((t) => t.date.isBefore(end) || t.date.isAtSameMomentAs(end))
        : where((t) => t.status)
            .where((t) => '${t.contactId}' == contactIdString);

    num totalSell = 0;
    num totalSellDeferred = 0;
    num totalSellCovered = 0;
    num totalSpend = 0;
    num totalSpendDeferred = 0;
    num totalSpendCovered = 0;
    num totalWage = 0;

    for (final ticket in filterByRange) {
      switch (ticket.ticketType) {
        case TicketType.sell:
          totalSell += ticket.total;

        case TicketType.sellDeferred:
          totalSellDeferred += ticket.total;

        case TicketType.sellCovered:
          totalSellCovered += ticket.total;

        case TicketType.spend:
          totalSpend += ticket.total;

        case TicketType.spendDeferred:
          totalSpendDeferred += ticket.total;

        case TicketType.spendCovered:
          totalSpendCovered += ticket.total;

        case TicketType.wage:
          totalWage += ticket.total;

        case TicketType.unknown:
          print(
              'rhaaaa unknow ticket type in herderTicketTypeRange, not again !');

        default:
          print(
              'default error in herderAllTicketTypeEndDate ${ticket.toJson()} !');
      }
    }

    return {
      TicketType.sell: totalSell,
      TicketType.sellDeferred: totalSellDeferred,
      TicketType.sellCovered: totalSellCovered,
      TicketType.spend: totalSpend,
      TicketType.spendDeferred: totalSpendDeferred,
      TicketType.spendCovered: totalSpendCovered,
      TicketType.wage: totalWage,
    };
  }

  num herderTicketsPurchasingPowerDateEnd(int contactId, DateTime endDate) {
    final Map<TicketType, num> herderAllTicketTypeEndDateMap =
        herderAllTicketTypeEndDate('$contactId', end: endDate);

    final sellDeferred =
        herderAllTicketTypeEndDateMap[TicketType.sellDeferred] ?? 0;
    final sellCovered =
        herderAllTicketTypeEndDateMap[TicketType.sellCovered] ?? 0;
    final spendCovered =
        herderAllTicketTypeEndDateMap[TicketType.spendCovered] ?? 0;
    final spendDeferred =
        herderAllTicketTypeEndDateMap[TicketType.spendDeferred] ?? 0;
    final wages = herderAllTicketTypeEndDateMap[TicketType.wage] ?? 0;

    final clientBalance = sellCovered - sellDeferred;
    final supplierBalance = spendCovered - spendDeferred;

    var _pp = (clientBalance - supplierBalance);
    _pp -= wages;
    //print(        '_pp $_pp = (client $clientBalance - supplier $supplierBalance) - wage $wages');
    return _pp;
  }

  num herderAllWages(int contactId, {DateTime? end}) => where((t) => t.status)
      .where((t) => t.contactId == contactId)
      .where((t) => t.date.isBefore(end ?? DateTime.now()))
      .where((t) => t.ticketType == TicketType.wage)
      .fold(0, (prev, e) => prev + e.received);
}

extension TicketsHerderPurchasingPowerDateRange on Iterable<TicketWeebi> {
  num herderPurchasingPowerDateRange(
      int contactId, DateTime previousMonthStart, DateTime previousMonthEnd) {
    final client = getClientBalanceThisMonthNoClosings(
        contactId, previousMonthStart, previousMonthEnd);
    final supplier = getSupplierBalanceThisMonthNoClosings(
        contactId, previousMonthStart, previousMonthEnd);
    final wage = wageThisMonth(contactId, previousMonthStart,
        previousMonthEnd); // changed from previousMonthStart
    final _pp = (client - supplier) - wage;
    // print('client $client supplier $supplier wage $wage');
    // print('_pp $_pp');
    return _pp;
  }

  num getClientBalanceThisMonthNoClosings(
      int contactId, DateTime dateMonthStart, DateTime dateMonthEnd) {
    final num clientDebit =
        clientSellDeferredThisMonth(contactId, dateMonthStart, dateMonthEnd);
    final num clientCredit =
        clientSellCoveredRange(contactId, dateMonthStart, dateMonthEnd);

    return clientCredit - clientDebit;
  }

  num clientSellDeferredThisMonth(
      int contactId, DateTime dateMonthStart, DateTime dateMonthEnd) {
    // print('dateMonthStart $dateMonthStart');
    // print('dateMonthEnd $dateMonthEnd');
    return where((t) => t.status)
        .where((t) => t.contactId == contactId)
        .where((t) =>
            t.date.isAfter(dateMonthStart) ||
            t.date.isAtSameMomentAs(dateMonthStart))
        .where((t) =>
            t.date.isBefore(dateMonthEnd) ||
            t.date.isAtSameMomentAs(dateMonthEnd))
        .where((t) => t.ticketType == TicketType.sellDeferred)
        .fold(0, (prev, e) => prev + e.totalPriceTaxAndPromoIncluded);
  }

  num clientSellCoveredRange(int contactId, DateTime start, DateTime end) =>
      where((t) => t.status)
          .where((t) => t.contactId == contactId)
          .where((t) => t.date.isAfter(start) || t.date.isAtSameMomentAs(start))
          .where((t) => t.date.isBefore(end) || t.date.isAtSameMomentAs(end))
          .where((t) => t.ticketType == TicketType.sellCovered)
          .fold(0, (prev, e) => prev + e.received);

  num getSupplierBalanceThisMonthNoClosings(
      int contactId, DateTime dateMonthStart, DateTime dateMonthEnd) {
    final supplierDebit =
        supplierSpendDeferredThisMonth(contactId, dateMonthStart, dateMonthEnd);
    final supplierCredit =
        supplierSpendCoveredThisMonth(contactId, dateMonthStart, dateMonthEnd);

    return supplierCredit - supplierDebit;
  }

  ///
  /// THIS MONTH
  ///
  num supplierSpendDeferredThisMonth(
          int contactId, DateTime dateMonthStart, DateTime dateMonthEnd) =>
      where((t) => t.status)
          .where((t) => t.contactId == contactId)
          .where((t) => t.ticketType == TicketType.spendDeferred)
          .where((t) =>
              t.date.isAfter(dateMonthStart) ||
              t.date.isAtSameMomentAs(dateMonthStart))
          .where((t) =>
              t.date.isBefore(dateMonthEnd) ||
              t.date.isAtSameMomentAs(dateMonthEnd))
          .fold(0, (prev, e) => prev + e.totalCostTaxAndPromoIncluded);
  // adding ?? to prevent total crash

  num supplierSpendCoveredThisMonth(
          int contactId, DateTime dateMonthStart, DateTime dateMonthEnd) =>
      where((t) => t.status)
          .where((t) => t.contactId == contactId)
          .where((t) => t.ticketType == TicketType.spendCovered)
          .where((t) =>
              t.date.isAfter(dateMonthStart) ||
              t.date.isAtSameMomentAs(dateMonthStart))
          .where((t) =>
              t.date.isBefore(dateMonthEnd) ||
              t.date.isAtSameMomentAs(dateMonthEnd))
          .fold(0, (prev, e) => prev! + e.received) ??
      0;

  num wageThisMonth(
          int contactId, DateTime dateMonthStart, DateTime dateMonthEnd) =>
      where((t) => t.status)
          .where((t) => t.contactId == contactId)
          .where((t) => t.ticketType == TicketType.wage)
          .where((t) =>
              t.date.isAfter(dateMonthStart) ||
              t.date.isAtSameMomentAs(dateMonthStart))
          .where((t) =>
              t.date.isBefore(dateMonthEnd) ||
              t.date.isAtSameMomentAs(dateMonthEnd))
          .fold(0, (prev, e) => prev! + e.received) ??
      0;
}

==> models_weebi-1.1.1.tar.gz: lib/src/extensions/tickets_purchasing_power.dart <==
// Project imports:

import 'package:models_weebi/src/common/ticket_type.dart';
import 'package:models_weebi/weebi_models.dart';

extension TicketsHerderPurchasingPowerEndDate on Iterable<TicketWeebi> {
  num herderTicketTypeEndDate(TicketType ticketType, String contactIdString,
      {DateTime? end}) {
    final filterByRange = (end != null)
        ? where((t) => t.status)
            .where((t) => '${t.contactId}' == contactIdString)
            .where((t) => t.date.isBefore(end) || t.date.isAtSameMomentAs(end))
        : where((t) => t.status)
            .where((t) => '${t.contactId}' == contactIdString);

    print('in filter by range');
    switch (ticketType) {
      case TicketType.sell:
        return filterByRange
            .where((t) => t.ticketType == TicketType.sell)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.sellDeferred:
        return filterByRange
            .where((t) => t.ticketType == TicketType.sellDeferred)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.sellCovered:
        return filterByRange
            .where((t) => t.ticketType == TicketType.sellCovered)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.spend:
        return filterByRange
            .where((t) => t.ticketType == TicketType.spend)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.spendDeferred:
        return filterByRange
            .where((t) => t.ticketType == TicketType.spendDeferred)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.spendCovered:
        return filterByRange
            .where((t) => t.ticketType == TicketType.spendCovered)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.wage:
        return filterByRange
            .where((t) => t.ticketType == TicketType.wage)
            .fold(0, (prev, element) => prev + element.total);
      case TicketType.unknown:
        print(
            'rhaaaa unknow ticket type in herderTicketTypeRange, not again !');
        return 0;
      default:
        return 0;
    }
  }

  num herderTicketTypeRangeDate(TicketType ticketType, String contactIdString,
      DateTime start, DateTime end) {
    final filterByRange = where((t) => t.status)
        .where((t) => '${t.contactId}' == contactIdString)
        .where((t) => t.date.isAfter(start) || t.date.isAtSameMomentAs(start))
        .where((t) => t.date.isBefore(end) || t.date.isAtSameMomentAs(end));

    switch (ticketType) {
      case TicketType.sell:
        return filterByRange
            .where((t) => t.ticketType == TicketType.sell)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.sellDeferred:
        return filterByRange
            .where((t) => t.ticketType == TicketType.sellDeferred)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.sellCovered:
        return filterByRange
            .where((t) => t.ticketType == TicketType.sellCovered)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.spend:
        return filterByRange
            .where((t) => t.ticketType == TicketType.spend)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.spendDeferred:
        return filterByRange
            .where((t) => t.ticketType == TicketType.spendDeferred)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.spendCovered:
        return filterByRange
            .where((t) => t.ticketType == TicketType.spendCovered)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.wage:
        return filterByRange
            .where((t) => t.ticketType == TicketType.wage)
            .fold(0, (prev, element) => prev + element.total);

      case TicketType.unknown:
        print(
            'rhaaaa unknow ticket type in herderTicketTypeRange, not again !');

        return 0;
      default:
        return 0;
    }
  }

  Map<TicketType, num> herderAllTicketTypeEndDate(String contactIdString,
      {DateTime? end}) {
    final filterByRange = (end != null)
        ? where((t) => t.status)
            .where((t) => '${t.contactId}' == contactIdString)
            .where((t) => t.date.isBefore(end) || t.date.isAtSameMomentAs(end))
        : where((t) => t.status)
            .where((t) => '${t.contactId}' == contactIdString);

    num totalSell = 0;
    num totalSellDeferred = 0;
    num totalSellCovered = 0;
    num totalSpend = 0;
    num totalSpendDeferred = 0;
    num totalSpendCovered = 0;
    num totalWage = 0;

    for (final ticket in filterByRange) {
      switch (ticket.ticketType) {
        case TicketType.sell:
          totalSell += ticket.total;

        case TicketType.sellDeferred:
          totalSellDeferred += ticket.total;

        case TicketType.sellCovered:
          totalSellCovered += ticket.total;

        case TicketType.spend:
          totalSpend += ticket.total;

        case TicketType.spendDeferred:
          totalSpendDeferred += ticket.total;

        case TicketType.spendCovered:
          totalSpendCovered += ticket.total;

        case TicketType.wage:
          totalWage += ticket.total;

        case TicketType.unknown:
          print(
              'rhaaaa unknow ticket type in herderTicketTypeRange, not again !');

        default:
          print(
              'default error in herderAllTicketTypeEndDate ${ticket.toJson()} !');
      }
    }

    return {
      TicketType.sell: totalSell,
      TicketType.sellDeferred: totalSellDeferred,
      TicketType.sellCovered: totalSellCovered,
      TicketType.spend: totalSpend,
      TicketType.spendDeferred: totalSpendDeferred,
      TicketType.spendCovered: totalSpendCovered,
      TicketType.wage: totalWage,
    };
  }

  num herderTicketsPurchasingPowerDateEnd(int contactId, DateTime endDate) {
    final Map<TicketType, num> herderAllTicketTypeEndDateMap =
        herderAllTicketTypeEndDate('$contactId', end: endDate);

    final sellDeferred =
        herderAllTicketTypeEndDateMap[TicketType.sellDeferred] ?? 0;
    final sellCovered =
        herderAllTicketTypeEndDateMap[TicketType.sellCovered] ?? 0;
    final spendCovered =
        herderAllTicketTypeEndDateMap[TicketType.spendCovered] ?? 0;
    final spendDeferred =
        herderAllTicketTypeEndDateMap[TicketType.spendDeferred] ?? 0;
    final wages = herderAllTicketTypeEndDateMap[TicketType.wage] ?? 0;

    final clientBalance = sellCovered - sellDeferred;
    final supplierBalance = spendCovered - spendDeferred;

    var pp = (clientBalance - supplierBalance);
    pp -= wages;
    //print(        '_pp $_pp = (client $clientBalance - supplier $supplierBalance) - wage $wages');
    return pp;
  }

  num herderAllWages(int contactId, {DateTime? end}) => where((t) => t.status)
      .where((t) => t.contactId == contactId)
      .where((t) => t.date.isBefore(end ?? DateTime.now()))
      .where((t) => t.ticketType == TicketType.wage)
      .fold(0, (prev, e) => prev + e.received);
}

extension TicketsHerderPurchasingPowerDateRange on Iterable<TicketWeebi> {
  num herderPurchasingPowerDateRange(
      int contactId, DateTime previousMonthStart, DateTime previousMonthEnd) {
    final client = getClientBalanceThisMonthNoClosings(
        contactId, previousMonthStart, previousMonthEnd);
    final supplier = getSupplierBalanceThisMonthNoClosings(
        contactId, previousMonthStart, previousMonthEnd);
    final wage = wageThisMonth(contactId, previousMonthStart,
        previousMonthEnd); // changed from previousMonthStart
    final pp = (client - supplier) - wage;
    // print('client $client supplier $supplier wage $wage');
    // print('_pp $_pp');
    return pp;
  }

  num getClientBalanceThisMonthNoClosings(
      int contactId, DateTime dateMonthStart, DateTime dateMonthEnd) {
    final num clientDebit =
        clientSellDeferredThisMonth(contactId, dateMonthStart, dateMonthEnd);
    final num clientCredit =
        clientSellCoveredRange(contactId, dateMonthStart, dateMonthEnd);

    return clientCredit - clientDebit;
  }

  num clientSellDeferredThisMonth(
      int contactId, DateTime dateMonthStart, DateTime dateMonthEnd) {
    // print('dateMonthStart $dateMonthStart');
    // print('dateMonthEnd $dateMonthEnd');
    return where((t) => t.status)
        .where((t) => t.contactId == contactId)
        .where((t) =>
            t.date.isAfter(dateMonthStart) ||
            t.date.isAtSameMomentAs(dateMonthStart))
        .where((t) =>
            t.date.isBefore(dateMonthEnd) ||
            t.date.isAtSameMomentAs(dateMonthEnd))
        .where((t) => t.ticketType == TicketType.sellDeferred)
        .fold(0, (prev, e) => prev + e.totalPriceTaxAndPromoIncluded);
  }

  num clientSellCoveredRange(int contactId, DateTime start, DateTime end) =>
      where((t) => t.status)
          .where((t) => t.contactId == contactId)
          .where((t) => t.date.isAfter(start) || t.date.isAtSameMomentAs(start))
          .where((t) => t.date.isBefore(end) || t.date.isAtSameMomentAs(end))
          .where((t) => t.ticketType == TicketType.sellCovered)
          .fold(0, (prev, e) => prev + e.received);

  num getSupplierBalanceThisMonthNoClosings(
      int contactId, DateTime dateMonthStart, DateTime dateMonthEnd) {
    final supplierDebit =
        supplierSpendDeferredThisMonth(contactId, dateMonthStart, dateMonthEnd);
    final supplierCredit =
        supplierSpendCoveredThisMonth(contactId, dateMonthStart, dateMonthEnd);

    return supplierCredit - supplierDebit;
  }

  ///
  /// THIS MONTH
  ///
  num supplierSpendDeferredThisMonth(
          int contactId, DateTime dateMonthStart, DateTime dateMonthEnd) =>
      where((t) => t.status)
          .where((t) => t.contactId == contactId)
          .where((t) => t.ticketType == TicketType.spendDeferred)
          .where((t) =>
              t.date.isAfter(dateMonthStart) ||
              t.date.isAtSameMomentAs(dateMonthStart))
          .where((t) =>
              t.date.isBefore(dateMonthEnd) ||
              t.date.isAtSameMomentAs(dateMonthEnd))
          .fold(0, (prev, e) => prev + e.totalCostTaxAndPromoIncluded);
  // adding ?? to prevent total crash

  num supplierSpendCoveredThisMonth(
          int contactId, DateTime dateMonthStart, DateTime dateMonthEnd) =>
      where((t) => t.status)
          .where((t) => t.contactId == contactId)
          .where((t) => t.ticketType == TicketType.spendCovered)
          .where((t) =>
              t.date.isAfter(dateMonthStart) ||
              t.date.isAtSameMomentAs(dateMonthStart))
          .where((t) =>
              t.date.isBefore(dateMonthEnd) ||
              t.date.isAtSameMomentAs(dateMonthEnd))
          .fold(0, (prev, e) => prev! + e.received) ??
      0;

  num wageThisMonth(
          int contactId, DateTime dateMonthStart, DateTime dateMonthEnd) =>
      where((t) => t.status)
          .where((t) => t.contactId == contactId)
          .where((t) => t.ticketType == TicketType.wage)
          .where((t) =>
              t.date.isAfter(dateMonthStart) ||
              t.date.isAtSameMomentAs(dateMonthStart))
          .where((t) =>
              t.date.isBefore(dateMonthEnd) ||
              t.date.isAtSameMomentAs(dateMonthEnd))
          .fold(0, (prev, e) => prev! + e.received) ??
      0;
}
